#!/bin/awk -f
#
# sastmain.awk -- main script for SAStangle
#############################################################################
# Copyright (C) 2006 Russell V. Lenth - University of Iowa
# Last revised: 2006.12.19
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##############################################################################


### Function to run a specified program and get the exit code
function runprog(prog, args) {
  cmd = prog " " args
  exitcode = system(cmd)
}

### Function to copy a file
function cp(source, target) {
  if (windows)
    system("cmd /c copy " source " " target)
  else
    system("cp -f " source " " target)
}

### Function to delete a file
function rm(file) {
  if (windows)
    system("cmd /c del " file)
  else
    system("rm -f " file)
}

### Function to rename a file
function mv(source, target) {
  if (windows) {
    system("cmd /c copy " source " " target)
    system("cmd /c del " source)
  }
  else {
    system("mv -f " source " " target)
  }
}



BEGIN {

# Default settings
  runsas = 0

# Parse any command-line options
  
  for (i=1; (i<=ARGC && !infile); i++) {
    if (ARGV[i] == "-s") 
      runsas = 1
    else if (substr(ARGV[i],1,1) == "-")
      print "Warning: Ignored unknown option " ARGV[i]
    else 
      infile = ARGV[i]
  }
  
  if (!infile) {
    print "Syntax: sastangle [-s] filename[.SAStex]"
    print "   -s : Run SAS on resulting .sas file"
    exit 1
  }
  


# Process the file --------------------------------

  dot = index(infile, ".")
  if (!dot) {
    base = infile
    infile = infile ".SAStex"
    ext = ".SAStex"
  }
  else {
    base = substr(infile, 1, dot-1)
    ext = substr(infile, dot)
  }

# Figure out what to do, based on ext
# No need to check for SAStex 
#  if (ext == ".SAStex") {
#    hasSAS = 1
#  }
  
  if (ext == ".Rtex" || ext == ".Stex" || ext == ".nw" || ext == ".Rnw" || ext == ".Snw") {
# --- Only S code present ---
    print "Tangling embedded S code ..."
    gsub("#1", base ext, stangle)
    runprog(stangle, "")
      exit exitcode    
  }
  
  else if (ext == ".RSAStex" || ext == ".RStex" || ext == ".SASRtex" || ext == ".SRtex") {
# --- Mixed SAS and S (LaTeX format) ---
    print "Tangling embedded S code ..."
    cp(infile, base ".Rtex")
    gsub("#1", base ".Rtex", stangle)
    runprog(stangle, "")
    rm(base ".Rtex")
  }
  else if (ext == ".nwSAS" || ext == ".SASnw") {
# --- Mixed SAS and S (noweb format) ---
    print "Tangling embedded S code ..."
    cp(infile, base ".Rnw")
    gsub("#1", base ".Rnw", stangle)
    runprog(stangle, "")
    rm(base ".Rnw")
  }
  
# --- If we got here, we should tangle SAS code ---  

  print "Tangling embedded SAS code ..."
  runprog(awk, "-f " swpath "sastsetup.awk -f " swpath "saswv1.awk " infile)
  if (exitcode) {
    print "Error in SASweave source file " infile
    exit 3
  }

  if (runsas) {
    print "Running SAS ..."
    runprog(sas, base ".sas")
    if (exitcode) {
      print "SAS error (code = " exitcode ") - see " base ".log for details"
      if (exitcode > 1) exit 4
    }
  }  

}

END {

  print "\nDone."

}

