#!/bin/awk -f

# saswmain.awk -- main script for SASweave
# 
#############################################################################
# Copyright (C) 2006 Russell V. Lenth - University of Iowa
# Last revised: 2007.05.30
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##############################################################################


### Function to run a specified program and get the exit code
function runprog(prog, args) {
  cmd = prog " " args
  exitcode = system(cmd)
}

### Function to copy a file
function cp(source, target) {
  if (windows)
    system("cmd /c copy " source " " target)
  else
    system("cp -f " source " " target)
}

### Function to delete a file
function rm(file) {
  if (windows)
    system("cmd /c del " file)
  else
    system("rm -f " file)
}

### Function to rename a file
function mv(source, target) {
  if (windows) {
    system("cmd /c copy " source " " target)
    system("cmd /c del " source)
  }
  else {
    system("mv -f " source " " target)
  }
}



BEGIN {

# Default settings
  cleanup = 2
  target = "pdf"

# Parse any command-line options
  
  for (i=1; (i<=ARGC && !infile); i++) {
    if      (ARGV[i] == "-c") cleanup = 1
    else if (ARGV[i] == "-l") cleanup = 0
    else if (ARGV[i] == "-g") cleanup = 2
    else if (ARGV[i] == "-p") target = "pdf"
    else if (ARGV[i] == "-t") target = "tex"
    else if (ARGV[i] == "-n") target = "nw"
    else if (ARGV[i] == "-r") target = "Rtex"
    else if (ARGV[i] == "-p") target = "pdf"
    else if (substr(ARGV[i],1,1) == "-")
      print "Warning: Ignored unknown option " ARGV[i]
    else infile = ARGV[i]
  }
  
  if (!infile) {
    print "Syntax: sasweave [-c -n -g] [-p -t -n -r] filename[.SAStex]"
    print "   [-c -g -l] -- [Clean up / keep graphs / leave all] intermediate files"
    print "   [-p -t -n -r] -- Make [.pdf / .tex / .nw / .Rtex] file"
    print "   Defaults are -g and -p"
    exit 1
  }
  


# Process the file --------------------------------

  dot = index(infile, ".")
  if (!dot) {
    base = infile
    infile = infile ".SAStex"
    ext = ".SAStex"
  }
  else {
    base = substr(infile, 1, dot-1)
    ext = substr(infile, dot)
  }


# Figure out what to do, based on ext
  if (ext == ".SAStex") {
    hasSAS = 1; hasR = 0
  }
  else if (ext == ".Rtex" || ext == ".Stex") {
    hasSAS = 0; hasR = 1
  }
  else if (ext == ".nw" || ext == ".Rnw" || ext == ".Snw") {
    hasSAS = 0; hasR = 1
  }
  else if (ext == ".RSAStex" || ext == ".RStex") {
    hasSAS = 2; hasR = 1; inttgt = ".Rtex"
  }
  else if (ext == ".nwSAS") {
    hasSAS = 2; hasR = 1; inttgt = ".Rnw"
  }
  else if (ext == ".SASRtex" || ext == ".SRtex") {
    hasSAS = 1; hasR = 2; inttgt = ".Rtex"
  }
  else if (ext == ".SASnw") {
    hasSAS = 1; hasR = 2; inttgt = ".Rnw"
  }
  else if (ext == ".tex") {
    hasSAS = 0; hasR = 0
  }
  
  if (hasSAS + hasR == 0 && target != "pdf") {
    print "There is nothing to do!"
    exit 99
  }
  
# ----- Run Sweave, if it is supposed to come before SASweave -----
  if (hasR == 1) {
    print "Running Sweave ..."
    if (hasSAS) {
      cp(infile, base inttgt)
      gsub (/#1/, base inttgt, sweave)
    }
    else {
      gsub (/#1/, infile, sweave)
    }
    runprog(sweave, "")
    if (exitcode)
      exit exitcode
    if (hasSAS) {
      mv(base ".tex", base ".SAStex")
      infile = base ".SAStex"
      if (cleanup)
        rm(base inttgt)      
    }
  }

# ----- Run SASweave, if called for
  if (hasSAS) {
  
    print "Pre-processing SASweave ..."
    runprog(awk, "-f " swpath "saswsetup.awk -f " swpath "saswv1.awk " infile)
    if (exitcode) {
      print "Error in SASweave source file " infile
      exit 3
    }

    print "Running SAS ..."
    runprog(sas, base ".sas")
    mv(base ".log", base ".saslog")
    if (exitcode) {
      print "SAS error (code = " exitcode ") - see " base ".saslog for details"
      if (exitcode > 1) exit 4
    }

    print "Post-processing ..."
    runprog(awk, "-f " swpath "saswv2.awk " base ".lst")
    if (exitcode) {
      print "Post-processing error - - report in detail to russell-lenth@uiowa.edu"
      exit 5
    }

    if (cleanup) {
      rm(base ".sas")
      rm(base ".saslog")
      rm(base ".lst")
      if (hasSAS==2) rm(base ".SAStex")
    }

# ----- Run Sweave, if called for secondary to SAS ----
    if (hasR == 2) {
      mv(base ".tex", base inttgt)
      gsub("#1", base inttgt, sweave)
      runprog(sweave, "")
      if (exitcode)
        exit exitcode
      if (cleanup) 
        rm(base inttgt)
    }
    
  }

  if (target != "pdf") {
    if (target != "tex")
      mv(base ".tex", base "." target)
    exit 0
  }

  runprog(pdflatex, base ".tex")
  if (exitcode) {
    exit 6
  }


  if (cleanup == 1) {
    if (hasSAS) rm(base "-swv*.pdf")
    if (hasR) rm(base "-*.pdf")
  }

  print "\nDone."

}

